import { Component, forwardRef } from "@angular/core";
import { Input, Output, EventEmitter } from "@angular/core";
import { FormControl } from "@angular/forms";

import { GeneralUtils } from "../service/general-utils.service";

import { NG_VALUE_ACCESSOR, ControlValueAccessor } from "@angular/forms";
import { ConfigurationEditor } from "../data/configuration-editor";
import * as Quill from "quill";

const noop = () => {};

export const CUSTOM_BITEDITOROLD_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => BitEditorOldComponent),
  multi: true
};

/**
 * Clase que utiliza un custom componente para campo editor de HTML
 * Utiliza la interfaz ControlValueAccessor
 *    <bit-editor [(ngModel)]="object.descripcion" [ngModelOptions]="{standalone: true}" height="250">&nbsp;&nbsp;Descripció</bit-editor>
 * Para utilizar, importar tambien js y css.
 *
 * https://www.primefaces.org/primeng/#/editor
 * https://quilljs.com/docs/modules/toolbar/
 *
 **/

@Component({
  selector: "bit-editor-old",
  template: ``,
  providers: [CUSTOM_BITEDITOROLD_CONTROL_VALUE_ACCESSOR]
})
export class BitEditorOldComponent implements ControlValueAccessor {
  @Input() control: FormControl;
  @Input() nombre: any;
  @Input() tipo: string = "default";
  // si es "custom", se utiliza el objeto de configuracion: configuration.
  // si es "default", no se utiliza ninguna configuracion (se muestran todos los controles)
  @Input() configuration: ConfigurationEditor = BitEditorOldComponent.globalConfiguration;
  @Input() placeholder: string = "";
  @Input() readOnly: boolean = false;
  @Input() height: number = 100;
  @Input() hideLabel: boolean = false; // forzar que no se reserve espacio para el nombre del campo
  @Input() private forceObligatorio: boolean = false;

  @Output() onChange = new EventEmitter<string>();

  public static globalConfiguration = ConfigurationEditor.newObject(); // es posible tener configuraciones globales en todos los
  // bit-editor de un proyecto
  private innerValue: any = "";
  private onTouchedCallback: () => void = noop;
  private onChangeCallback: (_: any) => void = noop;

  constructor() {}

  //get accessor
  get value(): any {
    return this.innerValue;
  }

  //set accessor including call the onchange callback
  set value(v: any) {
    if (v !== this.innerValue) {
      this.innerValue = v;
      this.onChangeCallback(v);
    }
  }

  get value_lectura(): string {
    if (this.value == null) {
      return "";
    } // si es null o undefined (es lo que evalúa el "==null")
    else {
      return this.value;
    }
  }

  changed(event) {
    if (event.htmlValue != this.innerValue) {
      this.onTouchedCallback();
      this.onChangeCallback(this.innerValue);
      this.onChange.emit(event);
    }
  }

  onFocus() {
    this.onTouchedCallback();
    // this.onChangeCallback(this.innerValue);
  }

  onInit(quill) {
    /*var toolbar = quill.editor.getModule('toolbar');
        toolbar.addHandler('link', function(value) {
            if (value) {
                var href = prompt('Introduzca la URL');
                this.quill.format('link', href);
            } else {
                this.quill.format('link', false);
            }
        });*/
  }

  writeValue(value: any) {
    if (value !== this.innerValue) {
      this.innerValue = value;
    }
  }

  registerOnChange(fn: any) {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any) {
    this.onTouchedCallback = fn;
  }

  // decidimos marcar el campo como obligatorio si se ha indicado a la fuerza (por parámetro)
  // o si el control de formulario asociado está saltando por ser obligatorio y no haberse rellenado
  get obligatorio(): boolean {
    return this.forceObligatorio || (this.control && this.control.errors && this.control.errors["required"]);
  }

  get configuracion(): ConfigurationEditor {
    return this.configuration ? this.configuration : BitEditorOldComponent.globalConfiguration;
  }
}
